@@ -61,7 +61,7 @@ main(argc, argv)
 	char *argv[];
 {
 	int	c, errflg, n, pid, bpflag, uid, truid, external, verbose;
-	int	speaksmtp, daemon, interactive, aliases, printq, dotiseof;
+	int	speaksmtp, daemon_flg, interactive, aliases, printq, dotiseof;
 	int	binary, vfd;
 	char	*ebp, *s, *cp, errbuf[BUFSIZ];
 	char	*from, *fullname, *mailbin, *mailshare, *mav[10];
@@ -87,7 +87,7 @@ main(argc, argv)
 	printq = strcmp(progname, MAILQ) == 0;
 	aliases = strcmp(progname, NEWALIASES) == 0;
 	from = fullname = NULL;
-	speaksmtp = daemon = interactive = 0;
+	speaksmtp = daemon_flg = interactive = 0;
 	bpflag = external = verbose = binary = 0;
 	dotiseof = 1;	/* tested below to see if changed */
 	ebp = errbuf;
@@ -122,7 +122,7 @@ main(argc, argv)
 				speaksmtp = 1;
 				break;
 			case 'd':	/* run as daemon */
-				daemon = 1;
+				daemon_flg = 1;
 				break;
 			case 't':	/* run in test mode */
 				interactive = 1;
@@ -218,7 +218,7 @@ main(argc, argv)
 		fprintf(stderr, "%s: ignored %s options:%s\n",
 				zmailer, progname, errbuf);
 otherprog:
-	if (speaksmtp + interactive + aliases + printq + daemon > 1) {
+	if (speaksmtp + interactive + aliases + printq + daemon_flg > 1) {
 		fprintf(stderr, "%s: conflicting sendmail options\n",
 				zmailer);
 	}
@@ -246,7 +246,7 @@ otherprog:
 		/* pass on suidness */
 		execl(path, "smtp-in", "-i", (char *)NULL);
 		perror(path);
-	} else if (daemon) {
+	} else if (daemon_flg) {
 		setuid(getuid());
 		if (chdir(mailbin) < 0) {
 			perror(mailbin);
