@@ -30,12 +30,20 @@ rfc822tz(ts, prettyname)
 	sign = offset >= 0;
 	if (!sign)
 	  offset = -offset;
-	sprintf(zone, "%c%02d%02d",
-		sign ? '-' : '+', offset / 60, offset % 60);
-	cp = zone;
+#ifdef  NO_TM_ZONE
 	if (prettyname)
-	  sprintf(cp+5," (%s)",ts->tm_zone);
-
+	  cp = timezone(tz.tz_minuteswest,ts->tm_isdst);
+#else   /* !NO_TM_ZONE */
+	if (prettyname)
+	  cp = ts->tm_zone;
+#endif        /* !NO_TM_ZONE */
+	else {
+	  sprintf(zone, "%c%02d%02d",
+		  sign ? '-' : '+', offset / 60, offset % 60);
+	  cp = zone;
+	  if (prettyname)
+	    sprintf(cp+5," (%s)",ts->tm_zone);
+	}
 
 #else	/* !USE_BSDTIMEZONE */
 #ifdef	USE_SYSVTIMEZONE
@@ -58,7 +66,7 @@ rfc822tz(ts, prettyname)
 	  strcat(zone+5, tzname[0 != daylight]);
 
 #else	/* !USE_SYSVTIMEZONE */
-#if 0
+#ifdef NO_TM_ZONE
 	cp = tzname[ts->tm_isdst];	/* this doesn't always work */
 #else
 	offset = ts->tm_gmtoff;
