@@ -85,7 +85,7 @@ struct spblk *
 sp_install(key, data, mark, q)
 	register spkey_t key;
 	register unsigned char *data;
-	register int mark;
+	register long mark;
 	register struct sptree *q;
 {
 	register struct spblk *n;
