@@ -206,7 +206,7 @@ register int timeout;
  fd_set wfds;
  register int r;
  register int saveerrno;
- void (*old_sig)();
+ void* (*old_sig)();
  char *retval;
  
 
@@ -255,7 +255,7 @@ register unsigned short remote;
  unsigned short rremote;
  register int fl;
  fd_set wfds;
- void (*old_sig)();
+ void* (*old_sig)();
  
  old_sig = (void*)signal(SIGPIPE, SIG_IGN);
  
