@@ -240,7 +240,8 @@ leaux(fcfd, path, srcstbufp)
 	struct stat *srcstbufp;
 {
 	FILE *fp;
-	int status, len, checknum, checkbin;
+	int status, len, checknum;
+	long checkbin;
 	u_char *table;
 	struct stat objstbuf;
 	char buf[80];
