@@ -157,7 +157,7 @@ int
 icprintSN(spl)
 	struct spblk *spl;
 {
-	fprintf(pcfp, "%s\t%d\n", pname(spl->key), (int)spl->data);
+	fprintf(pcfp, "%s\t%ld\n", pname(spl->key), (long)spl->data);
 	return 0;
 }
 
