@@ -16,6 +16,7 @@
 #ifdef	USE_SYSLOG
 #include <syslog.h>
 #endif	/* USE_SYSLOG */
+#include <unistd.h>
 
 #include "prototypes.h"
 
@@ -39,7 +40,6 @@
 #ifdef	R_OK
 #undef	R_OK
 #endif	/* R_OK */
-#include <unistd.h>
 
 #endif	/* USE_LOCKF */
 
@@ -421,7 +421,7 @@ int	isRecpntAddr = 0;
 	for (h = e->e_headers; h != NULL; h = h->h_next) \
 		if ((e->e_resent == 0 || h->h_descriptor->class == Resent) \
 		    && h->h_descriptor->hdr_name != NULL \
-		    && CISTREQ(h->h_descriptor->hdr_name + e->e_resent, X)) \
+		    && STREQ(h->h_descriptor->hdr_name + e->e_resent, X)) \
 			break;
 
 /* we insert the new header just before this point */
