@@ -126,6 +126,7 @@ static int syncweb __((struct dirqueue *dq, struct config_entry *ce));
 
 int global_maxkids = 1000;
 time_t now;
+FILE *statuslog = NULL;
 
 int
 main(argc, argv)
@@ -163,7 +164,7 @@ main(argc, argv)
 	daemonflg = 1;
 	dlyverbose = 0;
 	verbose = errflg = version = 0;
-	while ((c = getopt(argc, argv, "divf:FL:N:P:Q:VW")) != EOF)
+	while ((c = getopt(argc, argv, "divf:Fl:L:N:P:Q:VW")) != EOF)
 		switch (c) {
 		case 'f':	/* override default config file */
 			config = optarg;
@@ -171,6 +172,13 @@ main(argc, argv)
 		case 'F':
 			freeze = 1;
 			break;
+		case 'l':
+			statuslog = fopen(optarg,"a");
+			if (!statuslog) {
+			  perror("Can't open statistics log file (-l)");
+			  exit(1);
+			}
+			break;
 		case 'L':	/* override default log file */
 			log = optarg;
 			break;
@@ -357,6 +365,9 @@ main(argc, argv)
 	    /* Directory scan time for new jobs .. */
 	    next_dirscan = now + sweepinterval;	/* 10 seconds interval */
 	    dirqueuescan(".", dq);
+	    if (dq->wrkcount > 100)
+	      next_dirscan = now + 60; /* Lots of work, pick new jobs
+					  less frequently */
 	  }
 
 	  if (now >= next_idlecleanup) {
@@ -365,6 +376,12 @@ main(argc, argv)
 	  }
 
 	  /* Submit one item from pre-scheduler-queue into the scheduler */
+	  if (dq->wrkcount > 100) {
+	    /* If more than 100 in queue, submit 20 in a burst.. */
+	    int i;
+	    for (i=0; i < 20; ++i)
+	      syncweb(dq, tp);
+	  }
 	  if (dq->wrkcount > 0)
 	    syncweb(dq, tp);
 
@@ -495,6 +512,8 @@ static int dirqueuescan(dir, dq)
 	modtime = stbuf.st_mtime;
 #endif
 
+if (verbose) { printf("dirqueuescan(dir='%s') ",dir); fflush(stdout); }
+
 	/* Some changes lately, open the dir and read it */
 
 	dirp = opendir(dir);
@@ -571,6 +590,8 @@ static int dirqueuescan(dir, dq)
 #endif
 	closedir(dirp);
 
+	if (verbose) printf("wrkcount=%d new=%d\n",dq->wrkcount,newents);
+
 	return newents;
 }
 
@@ -703,6 +724,7 @@ slurp(fd, ino)
 	cfp->vfp = NULL;
 	cfp->head = NULL;
 	cfp->uid = stbuf.st_uid;
+	cfp->envctime = stbuf.st_ctime;
 	cfp->mark = V_NONE;
 	cfp->haderror = 0;
 	cfp->contents = contents;
