@@ -15,7 +15,7 @@
 #include <stdlib.h>
 
 /*
-   Each vertex arrives into SOME thread, which each belong of
+   Each vertex arrives into SOME thread, each of which belong to
    SOME thread-group, among which groups the transport channel
    programs started for any member thread can be shared among
    their members.
@@ -173,10 +173,11 @@ struct vertex *vtx;
 	thr->lastvertex = vtx;
 	thr->jobs       = 1;
 	vtx->thread     = thr;
-	if (verbose) {
-	  thr->channel    = strdup(vtx->orig[L_CHANNEL]->name);
-	  thr->host       = strdup(vtx->orig[L_HOST   ]->name);
-	}
+	thr->channel    = strdup(vtx->orig[L_CHANNEL]->name);
+	thr->host       = strdup(vtx->orig[L_HOST   ]->name);
+
+	if (verbose) printf("create_thread(%s/%d/%s) -> 0x%lx\n",
+			    thr->channel,thgrp->withhost,thr->host,(long)thr);
 
 	_thread_timechain_append(thr);
 
@@ -190,13 +191,15 @@ struct thread *thr;
 	/* Unlink this thread from thread-chain, and thread
 	   group.  Decrement thread-group count		    */
 
-	struct threadgroup *thg;
+	struct threadgroup *thg = thr->thgrp;
+	int last = 0;
 
-	if (verbose) {
-	  printf("delete_thread(%s/%s)\n",thr->channel,thr->host);
-	  free(thr->channel);
-	  free(thr->host);
-	}
+	if (verbose)
+	  printf("delete_thread(0x%lx:%s/%s) (thg=0x%lx)\n",
+		 (long)thr,thr->channel,thr->host,(long)thg);
+
+	free(thr->channel);
+	free(thr->host);
 
 	if (thr->jobs > 0) {
 	  if (verbose)
@@ -207,39 +210,56 @@ struct thread *thr;
 	/* Unlink us from the thread time-chain */
 	_thread_timechain_unlink(thr);
 
+	if (thr->nextthg == thr) { /* we are the last! */
+	  last = 1;
+	}
+       
 	/* ... and now from thread-group-ring */
 	thr->nextthg->prevthg = thr->prevthg;
 	thr->prevthg->nextthg = thr->nextthg;
 
-	thg = thr->thgrp;
-
 	/* If we are the ring hookup, connect another */
 	if (thg->thread == thr)
 	  thg->thread = thr->nextthg;
 
 	thg->threads -= 1;
 	/* If threads count goes zero.. */
-	if (thg->threads == 0) {
+	if (last) {
 	  thg->thread = NULL;	/* Oops.. we were last!		*/
 	  if (thr->proc &&
 	      thr->proc->thread == thr) { /* There is a process, idle it! */
 	    thr->proc->thread = NULL;
 	    thr->proc->vertex = NULL;
-	    thr->proc->next = thg->idleproc;
-	    thg->idleproc   = thr->proc;
-	    thg->idlecnt   += 1;
+	    thr->proc->next   = thg->idleproc;
+	    thg->idleproc     = thr->proc;
+	    /* thr->proc      = NULL; */ /* The `thr' goes into `free()' */
+	    thg->idlecnt     += 1;
 	    ++idleprocs;
+	  } else if (thr->proc != NULL &&
+		     thr->proc->pid <= 0) {
+	    thr->proc->thread = NULL;
+	    thr->proc->vertex = NULL;
+	  } else {
+	    fprintf(stderr,"delete_thread(1) thr->proc=0x%lx\n",(long)thr->proc);
 	  }
 	} else {
 	  /* Some threads left						*/
 	  /* If there is a process, pick another job..  It may be that
-	     the process' thread is no longer in a ring!		*/
+	     the process' current thread is no longer in the ring!	*/
 	  if (thr->proc &&
 	      thr->proc->thread == thr) {
 	    /* Set it into a new thread in our ring */
 	    thr->proc->vertex = thg->thread->vertices;
 	    thr->proc->thread = thg->thread;
 	    pick_next_vertex(thr->proc);
+	  } else if (thr->proc != NULL &&
+		     thr->proc->pid <= 0) {
+	    thr->proc->thread = NULL;
+	    thr->proc->vertex = NULL;
+	    fprintf(stderr, "delete_thread(2) thr->proc=0x%lx pid=%d\n",
+		    (long)thr->proc, thr->proc->pid);
+	  } else {
+	    fprintf(stderr, "delete_thread(3) thr->proc=0x%lx\n", (long)thr->proc);
 	  }
 	}
 
@@ -292,14 +312,15 @@ int cfgid;
 	struct web *wc = vp->orig[L_CHANNEL];
 	struct web *wh = vp->orig[L_HOST];
 
-if (verbose) printf("thread_linkin([%s/%s],...,%d)\n",wc->name,wh->name,cfgid);
+if (verbose) printf("thread_linkin([%s/%s],%s/%d/%s,%d)\n",wc->name,wh->name,
+		    cep->channel,cep->withhost,cep->host,cfgid);
 
 	/* char const *vp_chan = wc->name; */
 	/* char const *vp_host = wh->name; */
 
-	if (thrg_root == NULL) {
-	  thg = create_threadgroup(cep, wc, wh, cep->withhost);
-	}
+	if (thrg_root == NULL)
+	  (void) create_threadgroup(cep, wc, wh, cep->withhost);
+
 	/*
 	 *  Search for matching config-entry, AND matching channel,
 	 *  AND matching host (depending how the thread-group formation
@@ -347,24 +368,27 @@ if (verbose) printf("thread_linkin([%s/%s],...,%d)\n",wc->name,wh->name,cfgid);
 	    /* We have matching channel, AND matching host */
 
 	    /* Link the vertex into this thread! */
-if (verbose) printf(" ... on thg=0x%lx[%s/%d/%s] added into existing thread [%s/%s] thr->jobs=%d\n",
+
+if (verbose) printf("thread_linkin() to thg=0x%lx[%s/%d/%s]; added into existing thread [%s/%s] thr->jobs=%d\n",
 		    (long)thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name,thr->jobs+1);
 
 	    _thread_linktail(thr,vp);
 	    thr->jobs += 1;
+	    /* Hookay..  Try to start it, in case it isn't yet runing */
 	    thread_start(thr);
 
 	    return;
 	  }
 	  /* No matching thread, however this GROUP matches (or does it?) */
 
-if (verbose) printf(" ... on thg=0x%lx[%s/%d/%s] created a new thread [%s/%s]\n",
+if (verbose) printf("thread_linkin() to thg=0x%lx[%s/%d/%s]; created a new thread [%s/%s]\n",
 		    (long)thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name);
 
 	  /* Add a new thread into this group */
 	  thr = create_thread(thg,vp);
+	  /* Try to start it too */
 	  thread_start(thr);
 	  return;
 	}
@@ -373,10 +397,10 @@ if (verbose) printf(" ... on thg=0x%lx[%s/%d/%s] created a new thread [%s/%s]\n"
 	thg = create_threadgroup(cep, wc, wh, cep->withhost);
 	thr = create_thread(thg,vp);
 
-if (verbose) printf(" ... on thg=0x%lx[%s/%d/%s] created a new thread group, and thread [%s/%s]\n",
+if (verbose) printf("thread_linkin() to thg=0x%lx[%s/%d/%s]; created a new thread group, and thread [%s/%s]\n",
 		    (long)thg,cep->channel,thg->withhost,cep->host,
 		    wc->name,wh->name);
-
+	/* Try to start it too */
 	thread_start(thr);
 }
 
@@ -451,10 +475,10 @@ struct vertex *vtx;
 	if (vtx->nextitem != NULL)
 	  vtx->nextitem->previtem = vtx->previtem;
 
-
-	if (thr && thr->vertices == vtx)
+	if (thr->vertices   == vtx)
 	  thr->vertices   = vtx->nextitem;
-	if (thr && thr->lastvertex == vtx)
+
+	if (thr->lastvertex == vtx)
 	  thr->lastvertex = vtx->previtem;
 
 	vtx->nextitem = NULL;
@@ -567,10 +591,9 @@ struct thread *thr;
 	struct threadgroup *thg = thr->thgrp;
 
 	if (verbose)
-	  fprintf(stderr,"thread_start(thr=%s/%d/%s) (dt=%d/%d)",
+	  fprintf(stderr,"thread_start(thr=%s/%d/%s) (dt=%d)",
 		  vp->orig[L_CHANNEL]->name, thg->withhost,
-		  vp->orig[L_HOST]->name,
-		  (int)(thr->wakeup-now),(int)(vp->wakeup-now));
+		  vp->orig[L_HOST]->name, (int)(thr->wakeup-now));
 
 	if (thr->proc != NULL &&
 	    thr->proc->thread == thr) {
@@ -583,6 +606,9 @@ struct thread *thr;
 	  struct procinfo *proc;
 
 	  /* Idle processor(s) exists, take the first of them */
+	  /* XXX: try to optimize by finding idle proc with matching
+	          channel & host from previous activity. If not found,
+		  pick any. */
 	  thr->proc     = thg->idleproc;
 	  proc          = thr->proc;
 	  thg->idleproc = proc->next;
@@ -596,10 +622,12 @@ struct thread *thr;
 	     can always switch over in between threads */
 	  proc->ch = thr->vertices->orig[L_CHANNEL];
 
-	  /* Get rid of the old host web */
-	  proc->ho->kids -= 1;
-	  if (proc->ho->kids == 0 && proc->ho->link == NULL)
-	    unweb(L_HOST,proc->ho);
+	  if (proc->ho != NULL) {
+	    /* Get rid of the old host web */
+	    proc->ho->kids -= 1;
+	    if (proc->ho->kids == 0 && proc->ho->link == NULL)
+	      unweb(L_HOST,proc->ho);
+	  }
 
 	  /* Move the kid to this host web */
 	  proc->ho = thr->vertices->orig[L_HOST];
@@ -661,9 +689,6 @@ struct thread *thr;
 			 thr->vertices->orig[L_CHANNEL],
 			 thr->vertices->orig[L_HOST]);
 
-	if (rc)
-	  thg->transporters += 1;
-
 	return rc;
 }
 
@@ -686,14 +711,18 @@ struct procinfo *proc;
 
 	thr  = proc->thread;
 
-if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx)\n",
-		    proc->tofd, (long)thr);
+if (verbose) printf("pick_next_vertex(proc->tofd=%d, thr=0x%lx, jobs=%d)\n",
+		    proc->tofd, (long)thr, thr ? thr->jobs : 0);
 
 	if (thr == NULL) return; /* WE ARE IDLE! */
 
+	if (proc->fed == 0) return; /* Current one has not been fed.. */
+	proc->fed = 0;
+
 	thr0 = thr;
 	thg  = thr->thgrp;
-	vtx  = proc->vertex;
+	vtx  = thr->vertices;
+	/* proc->vertex->proc = NULL; */ /* Mark that we are busy.. */
 
 	/* Ok, if that one was/is busy/marked off, try next */
 	while (vtx) {
@@ -951,18 +980,23 @@ if (verbose) printf("idle_cleanup()\n");
 
 		/* Close the command channel, let it die itself.
 		   Rest of the cleanup happens via mux() service. */
-if (verbose) printf("idle_cleanup() killing processor on tofd=%d\n",p->tofd);
+		if (verbose)
+		  printf("idle_cleanup() killing TA on tofd=%d pid=%d\n",
+			 p->tofd,p->pid);
 		write(p->tofd,"\n",1);
 		close(p->tofd);
 		p->tofd = -1;
 		thg->idlecnt -= 1;
 		idleprocs    -= 1;
 		++freecount;
+		/* The thread-group can be deleted before reclaim() runs! */
 		thg->transporters -= 1;
+		p->thg = NULL;
 
-		/* now remove this entry from the chains */
+		/* Remove this entry from the chains */
 		*pp = p->next;
 		p = p->next;
+
 	      } else {
 		/* Move to the next possible idle process */
 		pp = &p->next;
@@ -991,31 +1025,45 @@ FILE *fp;
 	  fprintf(fp,"No threads/processes.\n");
 	  return;
 	}
+
+	time(&now);
+
 	for (thg = thrg_root;
 	     thg_once || thg != thrg_root;
 	     thg = thg->next) {
 	  int cnt, procs;
 	  struct procinfo *p = thg->idleproc;
-	  struct thread *thr = thg->thread;
+	  struct thread *thr;
 	  int thr_once = 1;
 
 	  thg_once = 0;
-	  fprintf(fp,"%s/%s/%d\t",
-		  thg->wchan->name, thg->cep->host, thg->withhost);
+	  fprintf(fp,"%s/%s/%d\n",
+		  thg->cep->channel, thg->cep->host, thg->withhost);
 	  cnt   = 0;
 	  procs = 0;
 	  thr_once = 1;
-	  for (; thr && (thr_once || thr != thg->thread); thr = thr->nexttr) {
+	  for (thr = thg->thread; thr && (thr_once || thr != thg->thread); thr = thr->nextthg) {
+	    thr_once = 0;
+
+	    fprintf(fp,"\t%s/%s/%d\tjobs=%d",
+		    /* thr->vertices->orig[L_CHANNEL]->name, */ thr->channel,
+		    /* thr->vertices->orig[L_HOST]->name, */ thr->host,
+		    thr->thgrp->withhost,thr->jobs);
 	    ++cnt;
 	    if (thr->proc != NULL &&
-		thr->proc->thread == thr) ++procs;
-	    thr_once = 0;
+		thr->proc->thread == thr) {
+	      ++procs;
+	      fprintf(fp," pid=%d",thr->proc->pid);
+	    }
+	    if (thr->wakeup > now)
+	      fprintf(fp," wakeup=%d",(int)(thr->wakeup - now));
+	    fprintf(fp,"\n");
 	  }
-	  fprintf(fp,"Threads: %d/%d ",thg->threads,cnt);
+	  fprintf(fp,"\t\tThreads: %d/%d ",thg->threads,cnt);
 	  fprintf(fp,"Procs: %d/%d ",thg->transporters,procs);
 	  cnt = 0;
 	  for (;p != 0; p = p->next) ++cnt;
 	  fprintf(fp,"Idle-procs: %d/%d\n",thg->idlecnt,cnt);
-	  
 	}
+	fprintf(fp,"Idleprocs: %d\n",idleprocs);
 }
