@@ -61,31 +61,38 @@ struct procinfo *proc;
 	char buf[500]; /* 60 or so SHOULD suffice.. */
 	int len, rc;
 
-	if (proc->vertex == NULL) return; /* Might be called without next process.. */
+	if (proc->thread == NULL) return; /* Might be called without next process.. */
 
 	if (proc->hungry) --hungry_childs;
+	/* It will be fed, clear this flag.. */
+	proc->hungry = 0;
 
 	proc->vertex->proc = proc; /* Flag that it is in processing */
 
-	if (proc->thread->thgrp->withhost)
+	if (proc->thg->withhost)
 	  sprintf(buf, "%s\n", proc->vertex->cfp->mid);
 	else
 	  sprintf(buf, "%s\t%s\n", proc->vertex->cfp->mid, proc->ho->name);
 
 	len = strlen(buf);
 
+	if (verbose)
+	  printf("feed: tofd=%d, len=%d, chan=%s, buf=%s",
+		 proc->tofd,len,proc->ch->name,buf);
+
 	rc = write(proc->tofd, buf, len);
+
 	if (rc != len) {
 	  /* Failed ?  How come ?
 	     Close the 'tofd' pipe, and clear proc ptr. */
 	  close(proc->tofd);
 	  proc->tofd = -1;
 	  proc->vertex->proc = NULL;
-	  return;
-	}
-
-	/* It is fed, clear this flag.. */
-	proc->hungry = 0;
+	  proc->vertex       = NULL;
+	  proc->thread->proc = NULL;
+	  proc->thread       = NULL;
+	} else
+	  proc->fed = 1;
 }
 
 /*
@@ -102,7 +109,7 @@ start_child(vhead, chwp, howp)
 	int	i;
 
 	if (verbose)
-	  printf("transport(vhead,chan=\"%s\",host=\"%s\")\n",
+	  printf("transport(vhead,chan=%s,host=%s)\n",
 		 chwp->name,howp->name);
 
 	if (vhead->ce.argv == NULL) {
@@ -248,8 +255,7 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	  i = scheduler_nofiles;
 	  cpids = (struct procinfo *)
 	    emalloc((unsigned)(i * sizeof (struct procinfo)));
-	  while (--i >= 0)
-	    cpids[i].pid = 0;
+	  memset(cpids, 0, sizeof(struct procinfo) * i);
 	}
 	proc = &cpids[fromfd];
 	proc->next   = NULL;
@@ -259,11 +265,14 @@ static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	proc->ho     = howp;
 	proc->vertex = vhead;
 	proc->thread = vhead->thread;
-	vhead->thread->proc = proc;
+	proc->thread->proc = proc;
+	proc->thg    = vhead->thread->thgrp;
+	proc->thg->transporters += 1;
 	proc->tofd   = tofd;
 	vhead->proc  = proc;
 	proc->carryover = NULL;
 	proc->hungry = 0;
+	proc->fed    = 0;
 #ifdef O_NONBLOCK /* POSIXy thing */
 	/* set up non-blocking I/O */
 	i = fcntl(fromfd, F_GETFL, 0);
@@ -320,14 +329,41 @@ if (verbose)
 	/* Reschedule the vertices that are left
 	   (that were not reported on).		*/
 
-	nvp = proc->vertex;
-	proc->vertex = NULL;
-	proc->thread = NULL;
-	for (vp = nvp; vp != NULL ; vp = nvp) {
-	  nvp = vp->nextitem;
-	  if (vp->ngroup == 0)
-	    abort();
-	  reschedule(vp, 0, -1);
+	/* ... but only if we were not in IDLE chain! */
+	if (proc->thread != NULL) {
+	  nvp = proc->thread->vertices;
+	  proc->thg->transporters -= 1;
+	  proc->vertex = NULL;
+	  proc->thread = NULL;
+	  proc->thg    = NULL;
+	  /* XX: thread reschedule ?? */
+	  for (vp = nvp; vp != NULL ; vp = nvp) {
+	    nvp = vp->nextitem;
+	    if (vp->ngroup == 0)
+	      abort();
+	    reschedule(vp, 0, -1);
+	  }
+	} else {
+	  /* We were in idle chain! */
+	  if (proc->thg != NULL /* And not killed by  idle_cleanup() */) {
+	    struct procinfo *p, **pp;
+	    p  =  proc->thg->idleproc;
+	    pp = &proc->thg->idleproc;
+
+	    while (p && p != proc) {
+		/* Move to the next possible idle process */
+		pp = &p->next;
+		p = p->next;
+	    }
+	    if (p == proc) {
+	      /* Remove this entry from the chains */
+	      *pp = p->next;
+	      p = p->next;
+	      proc->thg->transporters -= 1;
+	    } else {
+	      fprintf(stderr, "reclaim(fd=%d) proc->thg=0x%lx, proc=0x%lx NOT IN IDLE CHAIN!\n", fromfd, (long)proc->thg, (long)proc);
+	    }
+	  }
 	}
 }
 
@@ -541,24 +577,25 @@ static void readfrom(fd)
 {
 	int	n, dontbreak, bufsize = 2048;
 	char	*cp, *pcp, *eobuf, *buf;
+	struct procinfo *proc = &cpids[fd];
 
 	dontbreak = 0;
 	cp = pcp = NULL;
 
 	buf = (char *)emalloc(bufsize);
 
-	if (cpids[fd].carryover != NULL) {
-	  int carrylen = strlen(cpids[fd].carryover);
+	if (proc->carryover != NULL) {
+	  int carrylen = strlen(proc->carryover);
 	  if (carrylen > bufsize) {
 	    while (carrylen > bufsize)
 	      bufsize += 1024;
 	    buf = erealloc(buf,bufsize);
 	  }
-	  strcpy(buf, cpids[fd].carryover);
+	  strcpy(buf, proc->carryover);
 	  cp = buf+strlen(buf);
 	  pcp = buf;
-	  free(cpids[fd].carryover);
-	  cpids[fd].carryover = NULL;
+	  free(proc->carryover);
+	  proc->carryover = NULL;
 	  dontbreak = 1;
 	}
 
@@ -574,11 +611,9 @@ static void readfrom(fd)
 	    if (*cp == '\n') {
 	      *cp = '\0';
 	      if (verbose)
-		fprintf(stderr, "%d processed: %s\n",
-			fd, pcp);
+		fprintf(stderr, "%d fd=%d processed: %s\n",
+			proc->pid,fd, pcp);
 	      update(fd,pcp);
-	      if (cpids[fd].hungry)
-		feed_child(&cpids[fd]);
 	      *cp = '_';
 	      pcp = cp + 1;
 	      dontbreak = 0;
@@ -618,9 +653,9 @@ static void readfrom(fd)
 	if (n == 0 || (n < 0 && errno != EWOULDBLOCK && errno != EAGAIN &&
 		       errno != EINTR)) {
 	  /*printf("about to call waitandclose(), n=%d, errno=%d\n",n,errno);*/
-	  if (cpids[fd].tofd >= 0)
-	    close(cpids[fd].tofd);
-	  cpids[fd].tofd = -1;
+	  if (proc->tofd >= 0)
+	    close(proc->tofd);
+	  proc->tofd = -1;
 	  waitandclose(fd);
 	}
 	/* fprintf(stderr, "n = %d, errno = %d\n", n, errno); */
@@ -632,14 +667,14 @@ static void readfrom(fd)
 	 * or else some report may get lost somewhere.
 	 */
 	if (dontbreak) {
-	  if (cpids[fd].pid != 0) {
-	    cpids[fd].carryover = emalloc(cp-pcp+1);
-	    memcpy(cpids[fd].carryover, pcp, cp-pcp);
-	    cpids[fd].carryover[cp-pcp] = '\0';
+	  if (proc->pid != 0) {
+	    proc->carryover = emalloc(cp-pcp+1);
+	    memcpy(proc->carryover, pcp, cp-pcp);
+	    proc->carryover[cp-pcp] = '\0';
 	  } else
 	    fprintf(stderr,
-		    "HELP! Lost %d bytes (n=%d/%d, off=%d): '%s'\n",
-		    cp - pcp, n, errno, pcp-buf, pcp);
+		    "HELP! Lost %ld bytes (n=%d/%d, off=%ld): '%s'\n",
+		    (long)(cp - pcp), n, errno, (long)(pcp-buf), pcp);
 	}
 	free(buf);
 }
@@ -724,20 +759,23 @@ int signum;
 #else
 	int statloc;
 #endif
+
+	for (;;) {
+
 #ifdef USE_UNIONWAIT
-	pid = wait3(&statloc,WNOHANG,NULL);
+	  pid = wait3(&statloc,WNOHANG,NULL);
 #else
 #ifdef	USE_WAITPID
-	pid = waitpid(-1,&statloc,WNOHANG);
+	  pid = waitpid(-1,&statloc,WNOHANG);
 #else
-	pid = wait(&statloc);
+	  pid = wait(&statloc);
 #endif
 #endif
-	if (pid <= 0) return;
+	  if (pid <= 0) return;
 
 
 #ifdef	WIFEXITED
-	if (WIFEXITED(statloc)) ok = 1;
+	  if (WIFEXITED(statloc)) ok = 1;
 #else
       ERROR:ERROR:ERROR
 #endif
@@ -750,22 +788,24 @@ int signum;
 	if (verbose)
 	  fprintf(stderr,"sig_chld() pid=%d, ok=%d\n",pid,ok);
 
-	if (ok && cpids != NULL) {
-	  /* Only EXIT and SIGxx DEATHS accepted */
+	  if (ok && cpids != NULL) {
+	    /* Only EXIT and SIGxx DEATHS accepted */
 
-	  for (i = scheduler_nofiles-1; i >= 0; --i) {
-	    if (cpids[i].pid == pid) {
-	      cpids[i].pid = -pid;	/* Mark it as reaped.. */
-	      ok = 0;
-	      break;
-	    }
-	    if (cpids[i].pid == -pid) {
-printf(" .. already reaped ??\n");
-	      cpids[i].pid = -pid;	/* Mark it as reaped.. */
-	      ok = 0;
-	      break;
+	    for (i = scheduler_nofiles-1; i >= 0; --i) {
+	      if (cpids[i].pid == pid) {
+		cpids[i].pid = -pid; /* Mark it as reaped.. */
+		ok = 0;
+		break;
+	      }
+	      if (cpids[i].pid == -pid) {
+		printf(" .. already reaped ??\n");
+		cpids[i].pid = -pid; /* Mark it as reaped.. */
+		ok = 0;
+		break;
+	      }
 	    }
 	  }
+
 	}
 
 	/* re-instantiate the signal handler.. */
