@@ -77,29 +77,34 @@ update(fd, diagnostic)
 	int	index;
 	struct vertex *vp;
 	struct diagcodes *dcp;
+	struct procinfo *proc = &cpids[fd];
 
 	if (*diagnostic == 0) {
 	  /* Lone newline.. */
-	  if (cpids[fd].tofd >= 0 &&
-	      cpids[fd].hungry == 0) {
-	    cpids[fd].hungry = 1;
-	    time(&cpids[fd].hungertime);
+	  if (proc->tofd >= 0 &&
+	      proc->hungry == 0) {
+	    proc->hungry = 1;
+	    time(&proc->hungertime);
 	    ++hungry_childs;
 	  }
-	  /* Pick next, but don't feed it (yet)! */
-	  pick_next_vertex(&cpids[fd]);
+	  /* Pick next, and feed it! */
+	  pick_next_vertex(proc);
+	  if (proc->hungry)
+	    feed_child(proc);
 	  return;
 	}
 	if (*diagnostic == '#' /* DEBUG comment.. */) {
 	  if (strncmp(diagnostic,"#hungry",7)==0) {
-	    if (cpids[fd].tofd >= 0 &&
-		cpids[fd].hungry == 0) {
-	      cpids[fd].hungry = 1;
-	      time(&cpids[fd].hungertime);
+	    if (proc->tofd >= 0 &&
+		proc->hungry == 0) {
+	      proc->hungry = 1;
+	      time(&proc->hungertime);
 	      ++hungry_childs;
 	    }
-	    /* Pick next, but don't feed it (yet)! */
-	    pick_next_vertex(&cpids[fd]);
+	    /* Pick next, and feed it! */
+	    pick_next_vertex(proc);
+	    if (proc->hungry)
+	      feed_child(proc);
 	    return;
 	  }
 	  fprintf(stderr, "%s DBGdiag: %s\n", timestring(), diagnostic);
@@ -447,6 +452,22 @@ static void vtxupdate(vp, index)
 	  }
 }
 
+
+static void logstat(fp,vp,reason)
+	FILE *fp;
+	struct vertex *vp;
+	char *reason;
+{
+	time(&now);
+	fprintf(fp,"%ld %s %ld %ld %s %s/%s\n",
+		vp->cfp->ctime, vp->cfp->mid,
+		vp->cfp->envctime - vp->cfp->ctime,
+		now - vp->cfp->envctime, reason,
+		vp->orig[L_CHANNEL]->name,vp->orig[L_HOST]->name);
+	fflush(fp);
+}
+
+
 static void expaux(vp, index, buf)
 	struct vertex *vp;
 	int index;
@@ -464,6 +485,9 @@ static void expaux(vp, index, buf)
 		vp->orig[L_CHANNEL]->name, vp->orig[L_HOST]->name,
 		vp->cfp->erroraddr == NULL ? "?" : vp->cfp->erroraddr, buf);
 
+	if (statuslog)
+	  logstat(statuslog,vp,"expire");
+
 	/* Delete this vertex from scheduling datasets */
 	vtxupdate(vp, index);
 }
@@ -524,6 +548,9 @@ static int u_ok(vp, index, inum, offset, notary, message)
 
 	/* XX: Save/process info regarding delivery receipts! */
 
+	if (statuslog)
+	  logstat(statuslog,vp,"ok");
+
 	/* Delete this vertex from scheduling datasets */
 	vtxupdate(vp, index);
 	return 1;
@@ -581,6 +608,10 @@ static int u_error(vp, index, inum, offset, notary, message)
 		  vp->cfp->contents + offset + 2 + _CFTAG_RCPTPIDSIZE,
 		  message);
 	}
+
+	if (statuslog)
+	  logstat(statuslog,vp,"error");
+
 	/* Delete this vertex from scheduling datasets */
 	vtxupdate(vp, index);
 	return 1;
