@@ -9,35 +9,45 @@
 #include <pwd.h>
 #include <grp.h>
 #include "scheduler.h"
+#include "prototypes.h"
 
-extern int rc_command(), rc_expiry(), rc_expform(), rc_group(), rc_interval();
-extern int rc_maxchannel(), rc_maxhost(), rc_maxta(), rc_retries(), rc_user();
-extern int rc_skew(), rc_byhost(), rc_bychannel(), rc_gangschedule();
-extern int rc_deliveryform();
+static void celink __((struct config_entry *, struct config_entry **, struct config_entry **));
+static int readtoken __((FILE *fp, char *buf, int buflen));
+static u_int parse_intvl __((char *string));
+
+static int rc_command __((char *key, char *arg, struct config_entry *ce));
+static int rc_expform __((char *key, char *arg, struct config_entry *ce));
+static int rc_expiry __((char *key, char *arg, struct config_entry *ce));
+static int rc_group __((char *key, char *arg, struct config_entry *ce));
+static int rc_interval __((char *key, char *arg, struct config_entry *ce));
+static int rc_maxchannel __((char *key, char *arg, struct config_entry *ce));
+static int rc_maxhost __((char *key, char *arg, struct config_entry *ce));
+static int rc_maxta __((char *key, char *arg, struct config_entry *ce));
+static int rc_retries __((char *key, char *arg, struct config_entry *ce));
+static int rc_user __((char *key, char *arg, struct config_entry *ce));
+static int rc_skew __((char *key, char *arg, struct config_entry *ce));
+static int rc_byhost __((char *key, char *arg, struct config_entry *ce));
+static int rc_bychannel __((char *key, char *arg, struct config_entry *ce));
+static int rc_gangschedule __((char *key, char *arg, struct config_entry *ce));
+static int rc_deliveryform __((char *key, char *arg, struct config_entry *ce));
 
-extern char *mailbin;
-extern char *progname;
-extern char *qdefaultdir;
-extern char *replchannel;
-extern char *replhost;
 extern char *strchr();
 extern char *strerror();
 extern char *strrchr();
 extern int cistrcmp();
 extern int errno;
-extern int readtoken();
-extern int verbose;
-extern int global_maxkids;
+
+extern char *mailbin;
+
 extern struct group *getgrnam();
 extern struct passwd *getpwnam();
 
 extern void die();
-extern void vtxdo();
 
 struct config_entry *default_entry = NULL;
 
 
-struct rckeyword {
+static struct rckeyword {
 	char	*name;
 	int	(*parsef)();
 } rckeys[] = {
@@ -161,7 +171,7 @@ vtxprint(vp)
 	printf("\tskew %d\n", ce->skew);
 }
 
-void
+static void
 celink(ce, headp, tailp)
 	struct config_entry *ce;
 	struct config_entry **headp, **tailp;
@@ -287,7 +297,7 @@ readconfig(file)
 	return errflag ? NULL : head;
 }
 
-int
+static int
 readtoken(fp, buf, buflen)
 	FILE *fp;
 	char *buf;
@@ -343,7 +353,7 @@ readtoken(fp, buf, buflen)
 	return rv;
 }
 
-u_int
+static u_int
 parse_intvl(string)
 	char *string;
 {
@@ -375,10 +385,11 @@ parse_intvl(string)
 	return intvl;
 }
 
-struct config_entry *rrcf_head;
+static struct config_entry *rrcf_head;
 
-int
-vtxredo(spl)
+static int vtxredo __((struct spblk *spl));
+
+static int vtxredo(spl)
         struct spblk *spl;
 {
         struct ctlfile *cfp = (struct ctlfile *)spl->data;
@@ -449,8 +460,7 @@ rereadconfig(head, file)
 	return head;
 }
 
-int
-rc_command(key, arg, ce)
+static int rc_command(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -499,8 +509,7 @@ rc_command(key, arg, ce)
 	return 0;
 }
 
-int
-rc_expform(key, arg, ce)
+static int rc_expform(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -508,8 +517,7 @@ rc_expform(key, arg, ce)
 	return 0;
 }
 
-int
-rc_expiry(key, arg, ce)
+static int rc_expiry(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -517,8 +525,7 @@ rc_expiry(key, arg, ce)
 	return 0;
 }
 
-int
-rc_group(key, arg, ce)
+static int rc_group(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -534,8 +541,7 @@ rc_group(key, arg, ce)
 	return 0;
 }
 
-int
-rc_interval(key, arg, ce)
+static int rc_interval(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -543,8 +549,7 @@ rc_interval(key, arg, ce)
 	return 0;
 }
 
-int
-rc_maxchannel(key, arg, ce)
+static int rc_maxchannel(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -554,8 +559,7 @@ rc_maxchannel(key, arg, ce)
 	return 0;
 }
 
-int
-rc_maxhost(key, arg, ce)
+static int rc_maxhost(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -565,8 +569,7 @@ rc_maxhost(key, arg, ce)
 	return 0;
 }
 
-int
-rc_maxta(key, arg, ce)
+static int rc_maxta(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -578,8 +581,7 @@ rc_maxta(key, arg, ce)
 	return 0;
 }
 
-int
-rc_retries(key, arg, ce)
+static int rc_retries(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -623,8 +625,7 @@ rc_retries(key, arg, ce)
 	return 0;
 }
 
-int
-rc_user(key, arg, ce)
+static int rc_user(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -640,8 +641,7 @@ rc_user(key, arg, ce)
 	return 0;
 }
 
-int
-rc_skew(key, arg, ce)
+static int rc_skew(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -656,8 +656,7 @@ rc_skew(key, arg, ce)
 	return 0;
 }
 
-int
-rc_byhost(key, arg, ce)
+static int rc_byhost(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -665,8 +664,7 @@ rc_byhost(key, arg, ce)
 	return 0;
 }
 
-int
-rc_bychannel(key, arg, ce)
+static int rc_bychannel(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -674,8 +672,7 @@ rc_bychannel(key, arg, ce)
 	return 0;
 }
 
-int
-rc_gangschedule(key, arg, ce)
+static int rc_gangschedule(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
@@ -683,8 +680,7 @@ rc_gangschedule(key, arg, ce)
 	return 0;
 }
 
-int
-rc_deliveryform(key, arg, ce)
+static int rc_deliveryform(key, arg, ce)
 	char *key, *arg;
 	struct config_entry *ce;
 {
