@@ -16,7 +16,15 @@
 #include NDIR_H
 #include "zmsignal.h"
 
-int scheduler_nofiles = -1; /* Will be filled below */
+#include "prototypes.h"
+
+static int scheduler_nofiles = -1; /* Will be filled below */
+static void dotransport __((char **, int, struct vertex *, char*, char*));
+static void runcommand __((char **, int, char **, struct vertex *, struct web *, struct web *));
+static void stashprocess __((int, int, int, struct web *, struct web *, struct vertex *));
+static void reclaim __((int));
+static void waitandclose __((int));
+static void readfrom __((int));
 
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
@@ -31,28 +39,12 @@ struct procinfo *cpids = NULL;
 
 int	numkids = 0;
 
-extern int transportmaxnofiles;
-extern int resources_query_pipesize();
 extern char *getzenv();
-extern char *progname;
-extern char *replchannel;
-extern char *replhost;
-extern char *vtx_to_dir();
 extern char *strchr();
 extern int cistrcmp(), ranny();
 extern int epipe();
 extern int errno;
-extern int querysocket;
-extern int subsubdirs;
-extern int verbose;
-extern time_t qipcretry, time();
-extern void dotransport();
-extern void qprint();
-static void readfrom __((int fd));
-extern void runcommand();
-extern void stashprocess();
-extern void unweb(), reschedule(), fifoschedule();
-extern void update __((int fd, char *diags));
+extern time_t time();
 
 
 /*
@@ -65,8 +57,7 @@ extern void update __((int fd, char *diags));
 static int t_n, t_nmax;
 static char **t_files;
 
-int
-tscan(spl)
+static int tscan(spl)
 	struct spblk *spl;
 {
 	struct ctlfile *cfp = (struct ctlfile *)spl->data;
@@ -85,14 +76,12 @@ tscan(spl)
 	  t_nmax = 0;
 	  /* Clear the markings.. */
 	  cfp->mark    = V_NONE;
-	  /* cfp->atspool = NULL; */
 	  return -1;
 	}
 
 	t_files[t_n++] = cfp->mid;
 	cfp->mark    = V_NONE;
 
-	/* cfp->atspool = NULL; */
 	return 0;
 }
 
@@ -116,8 +105,7 @@ transport(vhead, channel, host)
 	dotransport(files, t_n, vhead, channel, host);
 }
 
-void
-dotransport(files, n, vhead, channel, host)
+static void dotransport(files, n, vhead, channel, host)
 	char *files[];
 	int n;
 	struct vertex *vhead;
@@ -143,10 +131,7 @@ dotransport(files, n, vhead, channel, host)
 	    return;		/* yes, this can happen */
 	}
 	if (vhead->ce.argv == NULL) {
-	  if (verbose) {
-	    fprintf(stderr, "No command defined for %s/%s!\n",
-		    channel, host);
-	  }
+	  fprintf(stderr, "No command defined for %s/%s!\n", channel, host);
 	  return;
 	}
 	/*
@@ -208,13 +193,12 @@ dotransport(files, n, vhead, channel, host)
 	  printf(" <<\\EOF\n");
 	  for (i = 0; i < n; ++i)
 	    printf("%s\n", files[i]);
-	  printf("\n\\EOF\n");
+	  printf("\\EOF\n");
 	}
 	runcommand(files, n, av, vhead, chwp, howp);
 }
 
-void
-runcommand(filearr, nelem, argv, vhead, chwp, howp)
+static void runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	char	*filearr[];
 	int	nelem;
 	char	*argv[];
@@ -260,18 +244,14 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 
 	  /* keep current stderr for child stderr */
 	  /* close all other open filedescriptors */
-#if	1
+
 	  /* ... if detach() did its job, there shouldn't be any! */
 	  /* ... no, the 'querysock' is there somewhere!   */
 	  if (scheduler_nofiles < 1)
 	    scheduler_nofiles = resources_query_nofiles();
 	  for (i = 3; i < scheduler_nofiles; ++i)
 	    close(i);
-#else
-	  close(to[1]);
-	  close(from[0]);
-	  /* we can ignore the stdouts from other children */
-#endif
+
 	  resources_limit_nofiles(transportmaxnofiles);
 	  setgid(gid);	/* Do GID setup while still UID 0..   */
 	  setuid(uid);	/* Now discard all excessive powers.. */
@@ -314,13 +294,13 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 	}
 }
 
-void
-stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
+static void stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	int pid, fromfd, tofd;
 	struct web *chwp, *howp;
 	struct vertex *vhead;
 {
 	int i;
+	struct procinfo *proc;
 
 	if (verbose)
 	  fprintf(stderr, "stashprocess(%d, %d, %d, %s, %s)\n",
@@ -335,15 +315,18 @@ stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 	  while (--i >= 0)
 	    cpids[i].pid = 0;
 	}
-	cpids[fromfd].pid    = pid;
-	cpids[fromfd].reaped = 0;
-	cpids[fromfd].ch     = chwp;
-	cpids[fromfd].ho     = howp;
-	cpids[fromfd].vertices = vhead;
-	cpids[fromfd].tofd   = tofd;
-	vhead->proc = &cpids[fromfd];
-	cpids[fromfd].carryover = NULL;
-	cpids[fromfd].hungry = 1;
+
+	proc = &cpids[fromfd];
+
+	proc->pid    = pid;
+	proc->reaped = 0;
+	proc->ch     = chwp;
+	proc->ho     = howp;
+	proc->vertices = vhead;
+	proc->tofd   = tofd;
+	vhead->proc  = proc;
+	proc->carryover = NULL;
+	proc->hungry = 1;
 #ifdef O_NONBLOCK /* POSIXy thing */
 	/* set up non-blocking I/O */
 	i = fcntl(fromfd, F_GETFL, 0);
@@ -361,39 +344,38 @@ stashprocess(pid, fromfd, tofd, chwp, howp, vhead)
 #endif	/* O_NONBLOCK  */
 }
 
-void
-reclaim(fromfd)
+static void reclaim(fromfd)
 	int fromfd;
 {
 	struct vertex *vp, *nvp;
 
-	cpids[fromfd].pid = 0;
-	cpids[fromfd].reaped = 0;
-	if (cpids[fromfd].carryover != NULL) {
+	struct procinfo *proc = &cpids[fromfd];
+
+	proc->pid = 0;
+	proc->reaped = 0;
+	if (proc->carryover != NULL) {
 	  fprintf(stderr, "%s: HELP! Lost %d bytes: '%s'\n",
-		  progname, strlen(cpids[fromfd].carryover),
-		  cpids[fromfd].carryover);
-	  free(cpids[fromfd].carryover);
-	  cpids[fromfd].carryover = NULL;
+		  progname, strlen(proc->carryover),
+		  proc->carryover);
+	  free(proc->carryover);
+	  proc->carryover = NULL;
 	}
-	if (cpids[fromfd].ch != NULL) {
-	  --cpids[fromfd].ch->kids;
-	  if (cpids[fromfd].ch->fifohead)
-	    fifoschedule(cpids[fromfd].ch);
-	  else if (cpids[fromfd].ch->kids == 0 &&
-		   cpids[fromfd].ch->link == NULL) {
-	    unweb(L_CHANNEL, cpids[fromfd].ch);
-	    cpids[fromfd].ch = NULL;
+	if (proc->ch != NULL) {
+	  --proc->ch->kids;
+	  if (proc->ch->fifohead)
+	    fifoschedule(proc->ch);
+	  else if (proc->ch->kids == 0 && proc->ch->link == NULL) {
+	    unweb(L_CHANNEL, proc->ch);
+	    proc->ch = NULL;
 	  }
 	}
-	if (cpids[fromfd].ho != NULL) {
-	  --cpids[fromfd].ho->kids;
-	  if (cpids[fromfd].ho->fifohead)
-	    fifoschedule(cpids[fromfd].ho);
-	  else if (cpids[fromfd].ho->kids == 0 &&
-		   cpids[fromfd].ho->link == NULL) {
-	    unweb(L_HOST, cpids[fromfd].ho);
-	    cpids[fromfd].ho = NULL;
+	if (proc->ho != NULL) {
+	  --proc->ho->kids;
+	  if (proc->ho->fifohead)
+	    fifoschedule(proc->ho);
+	  else if (proc->ho->kids == 0 && proc->ho->link == NULL) {
+	    unweb(L_HOST, proc->ho);
+	    proc->ho = NULL;
 	  }
 	}
 	close(fromfd);
@@ -401,8 +383,8 @@ reclaim(fromfd)
 	/* Reschedule the vertices that are left
 	   (that were not reported on).		*/
 
-	nvp = cpids[fromfd].vertices;
-	cpids[fromfd].vertices = NULL;
+	nvp = proc->vertices;
+	proc->vertices = NULL;
 	for (vp = nvp; vp != NULL ; vp = nvp) {
 	  nvp = vp->nextitem;
 	  if (vp->ngroup == 0)
@@ -411,8 +393,7 @@ reclaim(fromfd)
 	}
 }
 
-void
-waitandclose(fd)
+static void waitandclose(fd)
 	int	fd;
 {
 #ifndef USE_SIGREAPER
@@ -517,7 +498,6 @@ time_t timeout;
 	struct sockaddr_in raddr;
 	int	raddrlen;
 	struct timeval tv;
-	time_t now;
 
 	time(&now);
 
@@ -566,6 +546,7 @@ time_t timeout;
 	      if (rc == 0) { /* Child! */
 		close(querysocket);
 		qprint(i);
+		execl("/bin/false",NULL);
 		_exit(0);
 	      }
 	      if (rc > 0)
@@ -588,72 +569,17 @@ time_t timeout;
 	return 0;
 }
 
-#if 0 /* ipcpause() NOT in use, all done now inside  mux() */
-void
-ipcpause(pollflag)
-int pollflag;
-{
-#ifdef	AF_INET
-	int	i, n, rc;
-	fd_set	mask;
-	struct sockaddr_in raddr;
-	int	raddrlen;
-	struct timeval timeout;
-
-	if (querysocket < 0 && !pollflag) {
-		pause();
-		return;
-	}
-	FD_ZERO(&mask);
-	FD_SET(querysocket, &mask);
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 0;
-	if (!pollflag)
-	  timeout.tv_sec = 9999; /* VERY long time..
-				    We wait for an interrupt.. */
-	if ((n = select(querysocket+1, &mask, NULL, NULL, &timeout)) < 0) {
-	  if (errno == EINTR)
-	    return;
-	} else if (n == 0) {
-	  /* fprintf(stderr, "abnormal 0 return from select!\n"); */
-	  return;
-	} else {
-	  raddrlen = sizeof raddr;
-	  i = accept(querysocket, (struct sockaddr *)&raddr, &raddrlen);
-	  if (i < 0) {
-	    perror("querysocket accept");
-	  } else {
-	    /* Fork for QPRINT */
-	    rc = fork();
-	    if (rc == 0) { /* Child! */
-	      close(querysocket);
-	      qprint(i);
-	      exit(0);
-	    }
-	    if (rc > 0)
-	      ++numkids;
-	    close(i);
-	  }
-	}
-#else	/* !AF_INET */
-	if (!pollflag)
-	  pause();
-#endif	/* AF_INET */
-}
-#endif /* ipcpause() NOT in use! */
-
 void
 queryipcinit()
 {
 #ifdef	AF_INET
-	time_t now;
 	struct servent *serv;
 	struct sockaddr_in sad;
 	int on = 1;
 
 	if (querysocket >= 0)
 		return;
-	now = time((time_t *)0);
+	time(&now);
 	if ((serv = getservbyname("mailq", "tcp")) == NULL) {
 	  fprintf(stderr, "No 'mailq' tcp service defined!\n");
 	  /* try again in 5 minutes or so */
@@ -691,7 +617,6 @@ mux(timeout)
 time_t timeout;
 {
 	int	fd;
-	time_t	now;
 
 	/*
 	 * Nice'n easy and simpleminded: grab a random file descriptor,
@@ -711,21 +636,13 @@ time_t timeout;
 	return 1;
 }
 
-ipcpause(pollflag)
-int pollflag;
-{
-	if (!pollflag)
-		pause();
-}
-
 void
 queryipcinit()
 {
 }
 #endif	/* HAVE_SELECT */
 
-static void
-readfrom(fd)
+static void readfrom(fd)
 	int fd;
 {
 	int	n, dontbreak, bufsize = 2048;
@@ -823,18 +740,6 @@ readfrom(fd)
 	free(buf);
 }
 
-#if 0
-close(fd)
-	int fd;
-{
-	fprintf(stderr, "closing fd %d\n", fd);
-	if (fd == querysocket && getppid() == 1)
-		abort();
-	xclose(fd);
-}
-#endif
-
-
 #if defined(USE_BINMKDIR) || defined(USE_BINRMDIR)
 
 /*
