@@ -2232,6 +2232,7 @@ mvdata(out,msg)
 	int has8bitsum = 0;
 	int from__err = 0;
 	int linecnt = 0;
+	int was8bit = 0;
 
 	idxnum = indexnum+1;
 
@@ -2253,6 +2254,8 @@ mvdata(out,msg)
 		/* An EOF in here is an error! */
 #if EOF != -1
 		if (c == EOF) return EOF;
+#else
+		if (c < 0) return EOF;
 #endif
 		if (--timercnt <= 0) {
 		  /* Re-arm for every kilobyte */
@@ -2260,22 +2263,28 @@ mvdata(out,msg)
 		  timercnt = 1024;
 		}
 		++cnt;
-		if (0x80 & c) has8bit = 1;
-		if ((state = sts[state+idxnum[c]]) & ~0xff) {
-			if (state & O_) {
-				if (state == endstate)
-					return cnt;
-				state = (char)state;
-				continue;
-			}
-			if (!ferror(out))
-				putc((state>>8), out);
-			if (col < (sizeof(linebuf)-1))
-				linebuf[col++] = (state >> 8);
-			state = (char)state;
+		if (0x80 & c) {
+		  if (!was8bit) putc(' ',out);
+		  fprintf(out,"=?UNKNOWN-8BIT?Q?=%02X?= ", c & 0xFF);
+		  was8bit = 1;
+		} else {
+		  was8bit = 0;
+		  if ((state = sts[state+idxnum[c]]) & ~0xff) {
+		    if (state & O_) {
+		      if (state == endstate)
+			return cnt;
+		      state = (char)state;
+		      continue;
+		    }
+		    if (!ferror(out))
+		      putc((state>>8), out);
+		    if (col < (sizeof(linebuf)-1))
+		      linebuf[col++] = (state >> 8);
+		    state = (char)state;
+		  }
+		  if (!ferror(out))
+		    putc(c, out);
 		}
-		if (!ferror(out))
-			putc(c, out);
 		if (col < (sizeof(linebuf)-1))
 			linebuf[col++] = c;
 		/* LF, or something else ?  Here the else.. */
