@@ -5,6 +5,9 @@
 
 #include <stdio.h>
 #include "hostenv.h"
+#ifdef linux
+#define __USE_BSD 1
+#endif
 #include <ctype.h>
 #include <pwd.h>
 #include <sysexits.h>
@@ -84,6 +87,12 @@ typedef char msgdata;
 #include "mail.h"
 #include "splay.h"
 
+#ifdef    USE_SETUIDX
+# include <sys/id.h>
+# define  USE_SETREUID
+# define  setreuid(x,y)   setuidx(ID_REAL|ID_EFFECTIVE, y)
+#endif
+
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
 #endif	/* USE_UNIONWAIT */
@@ -181,14 +190,30 @@ main(argc, argv)
 	  exit(EX_USAGE);
 	}
 
-	while (!getout && fgets(file, sizeof file, stdin)) {
+	while (!getout) {
+	  char *s;
+	  /* Input:
+	       spool/file/name [ \t host.info ] \n
+	   */
+
+	  printf("#hungry\n");
+	  fflush(stdout);
+	  if (fgets(file, sizeof file, stdin) == NULL)
+	    break;
 	  if (emptyline(file, sizeof file))
 	    break;
+
+	  s = strchr(file,'\t');
+	  if (s != NULL) {
+	    /* Ignore the host part */
+	    *s = 0;
+	  }
+
 	  dp = ctlopen(file, channel, (char *)NULL, &getout, NULL, NULL);
-	  if (dp == NULL)
-	    continue;
-	  process(dp);
-	  ctlclose(dp);
+	  if (dp != NULL) {
+	    process(dp);
+	    ctlclose(dp);
+	  }
 	}
 	exit(0);
 	/* NOTREACHED */
