@@ -199,7 +199,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	char *mfpath;
 	int i, n;
 	struct address *ap;
-	struct rcpt *rp;
+	struct rcpt *rp = NULL;
 	struct stat stbuf;
 	char ***msgheaders = NULL;
 	char ***msgheaderscvt = NULL;
@@ -403,7 +403,6 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  case _CF_MSGHEADERS:
 	    {
 	      char **msgheader = NULL;
-	      char ***mmp;
 	      char *ss;
 	      int  headerlines = 0;
 	      int  headerspace = 0;
@@ -527,7 +526,7 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 #ifdef	USE_MMAP
 	d.let_buffer = (char *)mmap(NULL, stbuf.st_size,
 				    PROT_READ, MAP_SHARED, d.msgfd, 0);
-	if ((int)d.let_buffer == -1) {
+	if ((long)d.let_buffer == -1L) {
 	  warning("Out of MMAP() memory! Tried to map in (r/o) %d bytes (%m)",
 		  stbuf.st_size);
 	  free(mfpath);
