@@ -158,7 +158,7 @@ main(argc, argv)
 	    channel = optarg;
 	    break;
 	  case 'h':		/* remote hostname */
-	    host = strsave(optarg);
+	    host = strdup(optarg);
 	    break;
 	  case 'Q':
 	    mimeqpnarrow = 1;
@@ -211,7 +211,7 @@ main(argc, argv)
 	  s = strchr(file,'\t');
 	  if (s != NULL) {
 	    if (host) free(host);
-	    host = strsave(s+1);
+	    host = strdup(s+1);
 	    *s = 0;
 	  }
 
