@@ -443,6 +443,8 @@ main(argc, argv)
 	} else
 	  logfp = NULL;
 
+	if (logfp) setvbuf(logfp, (char *)NULL, _IOLBF, 0);
+
 	getnobody();
 	getdaemon();
 
@@ -460,6 +462,8 @@ main(argc, argv)
 	  printf("#hungry\n");
 	  fflush(stdout);
 
+	  if (statusreport) report("#hungry");
+
 	  if (fgets(file, sizeof file, stdin) == NULL) break;
 
 	  if (emptyline(file, sizeof file))
@@ -905,6 +909,8 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	alarm(timeout);
 	gotalarm = 0;
 
+#if 0 /* This is broken - the lseek() below broke it, but it fixed a lot
+	 of other problems..  Thus possible 'cache' goes away.. */
 #ifndef	USE_MMAP
 	/* can we use cache of message body data */
 	if (convertmode == _CONVERT_NONE &&
@@ -925,7 +931,9 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	  if (fflush(fp) != 0)
 	    return EX_IOERR;
 	}
-#endif
+#endif /* !USE_MMAP */
+#endif /* 0 */
+
 	if (lseek(dp->msgfd, dp->msgbodyoffset, SEEK_SET) < 0L)
 	  warning("Cannot seek to message body! (%m)", (char *)NULL);
 
@@ -1697,6 +1705,10 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	if (logfp)
 	  fprintf(logfp, "%d#\t(Connecting to `%s' [%s] %24.24s)\n",
 		  pid, hostname, ipaddress, ctime(&now));
+	if (statusreport) {
+	  strcpy(remotehost,hostname);
+	  report("connecting to [%s]",ipaddress);
+	}
 	/* try grabbing a port */
 	sad.sin_family = AF_INET;
 	sad.sin_addr.s_addr = 0;
@@ -1765,6 +1777,8 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	se = strerror(errno);
 	sprintf(remotemsg, "500 (connect to %s [%s]: %s)",
 		hostname, ipaddress, se);
+	if (statusreport)
+	  report("%s",remotemsg+4);
 	notaryreport(NULL,NULL,remotemsg);
 
 	if (conndebug)
@@ -1772,7 +1786,7 @@ vcsetup(ipaddrcp, ipaddrlen, fdp, hostname, verboselog)
 	if (verboselog)
 		fprintf(verboselog,"%s\n",remotemsg);
 	if (logfp)
-	  fprintf(logfp,"%d#\t(connect failed to `%s' [%s]: %s)\n", pid, hostname, ipaddress, se);
+	  fprintf(logfp,"%d#\t%s\n", pid, remotemsg+4);
 	switch (errnosave) {	/* from sendmail... */
 	case EISCONN:
 	case ETIMEDOUT:
@@ -2176,7 +2190,7 @@ report(va_alist)
 	char *cp, buf[8192];
 	
 	va_start(ap);
-	sprintf(buf, "-%s <%ld ", remotehost, atol(msgfile));
+	sprintf(buf, "-%s ", remotehost);
 #ifdef	notdef
 	if (logfp)
 	  sprintf(buf+strlen(buf), ">>%s ", logfile);
